@@ -16,6 +16,7 @@
 
 from cm_api.endpoints.types import *
 from cm_api.endpoints import services, parcels, host_templates
+from sys import api_version
 
 __docformat__ = "epytext"
 
@@ -77,10 +78,14 @@ class ApiCluster(BaseApiResource):
   _ATTRIBUTES = {
     'name'              : None,
     'displayName'       : None,
+    'clusterUrl'        : None,
     'version'           : None,
     'fullVersion'       : None,
+    'hostsUrl'          : ROAttr(),
     'maintenanceMode'   : ROAttr(),
     'maintenanceOwners' : ROAttr(),
+    'entityStatus'      : ROAttr(),
+    'uuid'              : ROAttr(),
   }
 
   def __init__(self, resource_root, name=None, version=None, fullVersion=None):
@@ -255,14 +260,34 @@ class ApiCluster(BaseApiResource):
     """
     return self._cmd('stop')
 
-  def restart(self):
+  def restart(self, restart_only_stale_services=None,
+    redeploy_client_configuration=None,
+    restart_service_names=None):
     """
     Restart all services in the cluster.
     Services are restarted in the appropriate order given their dependencies.
 
+    @param restart_only_stale_services: Only restart services that have stale
+                                        configuration and their dependent
+                                        services. Default is False.
+    @param redeploy_client_configuration: Re-deploy client configuration for
+                                          all services in the cluster. Default
+                                          is False.
+    @param restart_service_names: Only restart services that are specified and their dependent services.
+                                  Available since API v11.
+    @since API v6
+
     @return: Reference to the submitted command.
     """
-    return self._cmd('restart')
+    if self._get_resource_root().version < 6:
+      return self._cmd('restart')
+    else:
+      args = dict()
+      args['restartOnlyStaleServices'] = restart_only_stale_services
+      args['redeployClientConfiguration'] = redeploy_client_configuration
+      if self._get_resource_root().version >= 11:
+        args['restartServiceNames'] = restart_service_names
+      return self._cmd('restart', data=args, api_version=6)
 
   def deploy_client_config(self):
     """
@@ -412,6 +437,58 @@ class ApiCluster(BaseApiResource):
 
     return self._cmd('rollingRestart', data=args, api_version=4)
 
+  def rolling_upgrade(self, upgrade_from_cdh_version,
+                      upgrade_to_cdh_version,
+                      upgrade_service_names,
+                      slave_batch_size=None,
+                      slave_fail_count_threshold=None,
+                      sleep_seconds=None):
+    """
+    Command to do a rolling upgrade of services in the given cluster
+
+    This command does not handle any services that don't support rolling
+    upgrades. The command will throw an error and not start if upgrade of
+    any such service is requested.
+
+    This command does not upgrade the full CDH Cluster. You should normally
+    use the upgradeCDH Command for upgrading the cluster. This is primarily
+    helpful if you need to need to recover from an upgrade failure or for
+    advanced users to script an alternative to the upgradeCdhCommand.
+
+    This command expects the binaries to be available on hosts and activated.
+    It does not change any binaries on the hosts.
+
+    @param upgrade_from_cdh_version: Current CDH Version of the services.
+           Example versions are: "5.1.0", "5.2.2" or "5.4.0"
+    @param upgrade_to_cdh_version: Target CDH Version for the services.
+           The CDH version should already be present and activated on the nodes.
+           Example versions are: "5.1.0", "5.2.2" or "5.4.0"
+    @param upgrade_service_names: List of specific services to be upgraded and restarted.
+    @param slave_batch_size: Number of hosts with slave roles to restart at a time
+           Must be greater than 0. Default is 1.
+    @param slave_fail_count_threshold: The threshold for number of slave host batches that
+           are allowed to fail to restart before the entire command is considered failed.
+           Must be >= 0. Default is 0.
+    @param sleep_seconds: Number of seconds to sleep between restarts of slave host batches.
+           Must be >=0. Default is 0.
+
+    @return: Reference to the submitted command.
+    @since: API v10
+    """
+    args = dict()
+    args['upgradeFromCdhVersion'] = upgrade_from_cdh_version
+    args['upgradeToCdhVersion'] = upgrade_to_cdh_version
+    args['upgradeServiceNames'] = upgrade_service_names
+
+    if slave_batch_size:
+      args['slaveBatchSize'] = slave_batch_size
+    if slave_fail_count_threshold:
+      args['slaveFailCountThreshold'] = slave_fail_count_threshold
+    if sleep_seconds:
+      args['sleepSeconds'] = sleep_seconds
+
+    return self._cmd('rollingUpgrade', data=args, api_version=10)
+
   def auto_assign_roles(self):
     """
     Automatically assign roles to hosts and create the roles for all the services in a cluster.
@@ -448,31 +525,129 @@ class ApiCluster(BaseApiResource):
     """
     return self._cmd('firstRun', None, api_version=7)
 
-  def upgrade_cdh(self, deploy_client_config=True, start_all_services=True, cdh_parcel_version=None):
+  def upgrade_cdh(self, deploy_client_config=True, start_all_services=True,
+      cdh_parcel_version=None, cdh_package_version=None,
+      rolling_restart=False, slave_batch_size=None, sleep_seconds=None,
+      slave_fail_count_threshold=None):
     """
-    Perform CDH upgrade to the next major version.
+    Perform CDH upgrade to the next major version. In v9+, also supports
+    minor CDH 5 upgrades (5.a.b to 5.x.y where x > a) and supports maintenance
+    release changes (a.b.x to a.b.y).
 
     If using packages, CDH packages on all hosts of the cluster must be
     manually upgraded before this command is issued.
- 
+
     The command will upgrade the services and their configuration to the
-    version available in the CDH5 distribution. All running services will
-    be stopped before proceeding.
+    requested version. All running services will be stopped before proceeding,
+    unless rolling restart is requested and is available.
 
     @param deploy_client_config: Whether to deploy client configurations
-           after the upgrade. Default is True.
+           after the upgrade. Default is True. Has no effect in v9+;
+           client configurations are always deployed.
     @param start_all_services: Whether to start all services after the upgrade.
-           Default is True.
-    @param cdh_parcel_version: If using parcels, the full version of an
-           already distributed parcel for the next major CDH version. Default
+           Default is True. Has no effect in v9+; services are always
+           restarted.
+    @param cdh_parcel_version: If upgrading to parcels, the full version of an
+           already distributed parcel for the next CDH version. Default
            is None. Example versions are: '5.0.0-1.cdh5.0.0.p0.11' or
-           '5.0.2-1.cdh5.0.2.p0.32'
+           '5.0.2-1.cdh5.0.2.p0.32'.
+    @param cdh_package_version: If upgrading to packages, the full version of an
+           already installed package for the next CDH version. Default
+           is None. Example versions are: '5.2.0' or '4.5.0'. Only available
+           since v9.
+    @param rolling_restart: If you'd like to do a rolling restart, set this to
+           True. Default is False. Only available since v9.
+    @param slave_batch_size: Controls the rolling restart slave batch size.
+           Only applicable when rolling_restart is True.
+    @param sleep_seconds: Controls how many seconds to sleep betweein rolling
+           restart batches. Only applicable when rolling_restart is True.
+    @param slave_fail_count_threshold: Controls how many slave restart failures
+           are tolerated in a rolling restart. Only applicable when
+           rolling_restart is True.
     @return: Reference to the submitted command.
-    @since: API v6
+    @since: API v6 for major upgrades only, v9 for maintenance and CDH 5 minor
+            releases.
     """
     args = dict()
     args['deployClientConfig'] = deploy_client_config
     args['startAllServices'] = start_all_services
     if cdh_parcel_version:
       args['cdhParcelVersion'] = cdh_parcel_version
+    if cdh_package_version:
+      args['cdhPackageVersion'] = cdh_package_version
+    if rolling_restart:
+      args['rollingRestartArgs'] = {
+          'slaveBatchSize' : slave_batch_size,
+          'sleepSeconds' : sleep_seconds,
+          'slaveFailCountThreshold' : slave_fail_count_threshold
+        }
     return self._cmd('upgradeCdh', data=args, api_version=6)
+
+  def configure_for_kerberos(self, datanode_transceiver_port=None,
+    datanode_web_port=None):
+    """
+    Command to configure the cluster to use Kerberos for authentication.
+
+    This command will configure all relevant services on a cluster for
+    Kerberos usage.  This command will trigger a GenerateCredentials command
+    to create Kerberos keytabs for all roles in the cluster.
+
+    @param datanode_transceiver_port: The HDFS DataNode transceiver port to use.
+           This will be applied to all DataNode role configuration groups. If
+           not specified, this will default to 1004.
+    @param datanode_web_port: The HDFS DataNode web port to use.  This will be
+           applied to all DataNode role configuration groups. If not specified,
+           this will default to 1006.
+    @return: Reference to the submitted command.
+    @since: API v11
+    """
+    args = dict()
+    if datanode_transceiver_port:
+        args['datanodeTransceiverPort'] = datanode_transceiver_port
+    if datanode_web_port:
+        args['datanodeWebPort'] = datanode_web_port
+    return self._cmd('configureForKerberos', data=args, api_version=11)
+
+  def export(self, export_auto_config=False):
+    """
+    Export the cluster template for the given cluster. ccluster must have host
+    templates defined. It cluster does not have host templates defined it will
+    export host templates based on roles assignment.
+
+    @param export_auto_config: Also export auto configured configs
+    @return: Return cluster template
+    @since: API v12
+    """
+
+    return self._get("export", ApiClusterTemplate, False,
+                     params=dict(exportAutoConfig=export_auto_config), api_version=12)
+
+  def pools_refresh(self):
+    """
+    Refresh Dynamic Pools configurations for relevant services..
+
+    @return: Reference to the submitted command.
+    @since: API v6
+    """
+    return self._cmd('poolsRefresh', api_version=6)
+
+  def list_dfs_services(self, view=None):
+    """
+    List available DFS (distributed file system) services in a cluster.
+    @param view: View to materialize
+    @return: List of available distributed file system services in the cluster.
+    @since: API v12
+    """
+    if view:
+      return self._get_resource_root().get("%s/%s?view=%s" % (self._path(), 'dfsServices', view))
+    else:
+      return self._get_resource_root().get("%s/%s" % (self._path(), 'dfsServices'))
+
+  def delete_credentials(self):
+    """
+    Delete kerberos credentials for services belonging to this cluster.
+
+    @return: Reference to the submitted command.
+    @since: API v18
+    """
+    return self._cmd('deleteCredentials', api_version=18)
