@@ -146,6 +146,24 @@ class ClouderaManager(BaseApiResource):
     """
     return self._cmd('importAdminCredentials', params=dict(username=username, password=password))
 
+  def get_licensed_feature_usage(self):
+    """
+    Retrieve a summary of licensed feature usage.
+
+    This command will return information about what Cloudera Enterprise
+    licensed features are in use in the clusters being managed by this Cloudera
+    Manager, as well as totals for usage across all clusters.
+
+    The specific features described can vary between different versions of
+    Cloudera Manager.
+
+    Available since API v6.
+    """
+    return self._get('getLicensedFeatureUsage',
+                     ret_type=ApiLicensedFeatureUsage,
+                     ret_is_list=False,
+                     api_version=6)
+
   def inspect_hosts(self):
     """
     Runs the host inspector on the configured hosts.
@@ -169,21 +187,40 @@ class ClouderaManager(BaseApiResource):
         'endTime': end_datetime.isoformat(),
         'includeInfoLog': includeInfoLog,
     }
-    return self._cmd('collectDiagnosticData', data=args)
+    # This method is deprecated as of CM API version 3 which was introduced
+    # in CM 4.5.
+    return self._cmd('collectDiagnosticData', data=args, api_version=2)
 
-  def collect_diagnostic_data_45(self, end_datetime, bundle_size_bytes, cluster_name=None):
+  def collect_diagnostic_data_45(self, end_datetime, bundle_size_bytes, cluster_name=None,
+                                 roles=None, collect_metrics=False, start_datetime=None):
     """
     Issue the command to collect diagnostic data.
+    If start_datetime is specified, diagnostic data is collected for the entire period between
+    start_datetime and end_datetime provided that bundle size is less than or equal to
+    bundle_size_bytes. Diagnostics data collection fails if the bundle size is greater than
+    bundle_size_bytes.
+
+    If start_datetime is not specified, diagnostic data is collected starting from end_datetime
+    and collecting backwards upto a maximum of bundle_size_bytes.
 
     @param end_datetime: The end of the collection period. Type datetime.
     @param bundle_size_bytes: The target size for the support bundle in bytes
     @param cluster_name: The cluster to collect or None for all clusters
+    @param roles: Role ids of roles to restrict log and metric collection to. Valid since v10.
+    @param collect_metrics: Whether to collect metrics for viewing as charts. Valid since v13.
+    @param start_datetime: The start of the collection period. Type datetime. Valid since v13.
     """
     args = {
         'endTime': end_datetime.isoformat(),
         'bundleSizeBytes': bundle_size_bytes,
         'clusterName': cluster_name
     }
+    if self._get_resource_root().version >= 10:
+      args['roles'] = roles
+    if self._get_resource_root().version >= 13:
+      args['enableMonitorMetricsCollection'] = collect_metrics
+      if start_datetime is not None:
+          args['startTime'] = start_datetime.isoformat()
     return self._cmd('collectDiagnosticData', data=args)
 
   def hosts_decommission(self, host_names):
@@ -197,6 +234,21 @@ class ClouderaManager(BaseApiResource):
     """
     return self._cmd('hostsDecommission', data=host_names)
 
+  def hosts_offline_or_decommission(self, host_names, timeout=None):
+    """
+    Decommission the specified hosts by offlining or decommissioning the slave
+    roles and stopping the remaining ones. Offline has precedence if supported
+    by the service.
+
+    @param host_names: List of names of hosts to be decommissioned.
+    @param timeout: Offline timeout in seconds. Omit to get the default timeout (4 hours).
+    @return: Information about the submitted command.
+    @since: API v17
+    """
+    return self._cmd('hostsOfflineOrDecommission', data=host_names,
+        params=timeout and dict(timeout=timeout) or dict(),
+        api_version=17)
+
   def hosts_recommission(self, host_names):
     """
     Recommission the specified hosts by recommissioning the slave roles.
@@ -208,6 +260,23 @@ class ClouderaManager(BaseApiResource):
     """
     return self._cmd('hostsRecommission', data=host_names)
 
+  def hosts_recommission_with_start(self, host_names):
+    """
+    Recommission the specified hosts by recommissioning the slave roles.
+    This command will start the roles before recommissioning.
+
+    Warning: Evolving. This method may change in the future and does not
+    offer standard compatibility guarantees.
+    Do not use without guidance from Cloudera.
+
+    Currently, only HDFS DataNodes will be started by this command.
+
+    @param host_names: List of names of hosts to be recommissioned.
+    @return: Information about the submitted command.
+    @since: API v15
+    """
+    return self._cmd('hostsRecommissionWithStart', data=host_names, api_version=15)
+
   def hosts_start_roles(self, host_names):
     """
     Start all the roles on the specified hosts.
@@ -218,7 +287,7 @@ class ClouderaManager(BaseApiResource):
     """
     return self._cmd('hostsStartRoles', data=host_names)
 
-  def create_peer(self, name, url, username, password):
+  def create_peer(self, name, url, username, password, peer_type="REPLICATION"):
     """
     Create a new peer for replication.
 
@@ -226,29 +295,46 @@ class ClouderaManager(BaseApiResource):
     @param url: The url of the peer.
     @param username: The admin username to use to setup the remote side of the peer connection.
     @param password: The password of the admin user.
+    @param peer_type: Added in v11. The type of the peer. Defaults to 'REPLICATION'.
     @return: The newly created peer.
     @since: API v3
     """
+    if self._get_resource_root().version < 11:
+      peer_type = None
     peer = ApiCmPeer(self._get_resource_root(),
         name=name,
         url=url,
         username=username,
-        password=password)
+        password=password,
+        type=peer_type)
     return self._post("peers", ApiCmPeer, data=peer, api_version=3)
 
-  def delete_peer(self, name):
+  def _get_peer_type_param(self, peer_type):
+    """
+    Checks if the resource_root's API version is >= 11 and construct type param.
+    """
+    params = None
+    if self._get_resource_root().version >= 11:
+      params = {
+        'type':   peer_type,
+      }
+    return params
+
+  def delete_peer(self, name, peer_type="REPLICATION"):
     """
     Delete a replication peer.
 
     @param name: The name of the peer.
+    @param peer_type: Added in v11. The type of the peer. Defaults to 'REPLICATION'.
     @return: The deleted peer.
     @since: API v3
     """
-    return self._delete("peers/" + name, ApiCmPeer, api_version=3)
+    params = self._get_peer_type_param(peer_type)
+    return self._delete("peers/" + name, ApiCmPeer, params=params, api_version=3)
 
   def update_peer(self,
       current_name,
-      new_name, new_url, username, password):
+      new_name, new_url, username, password, peer_type="REPLICATION"):
     """
     Update a replication peer.
 
@@ -257,15 +343,19 @@ class ClouderaManager(BaseApiResource):
     @param new_url: The new url for the peer.
     @param username: The admin username to use to setup the remote side of the peer connection.
     @param password: The password of the admin user.
+    @param peer_type: Added in v11. The type of the peer. Defaults to 'REPLICATION'.
     @return: The updated peer.
     @since: API v3
     """
+    if self._get_resource_root().version < 11:
+      peer_type = None
     peer = ApiCmPeer(self._get_resource_root(),
         name=new_name,
         url=new_url,
         username=username,
-        password=password)
-    return self._put("peers/" + current_name, data=peer, api_version=3)
+        password=password,
+        type=peer_type)
+    return self._put("peers/" + current_name, ApiCmPeer, data=peer, api_version=3)
 
   def get_peers(self):
     """
@@ -276,25 +366,29 @@ class ClouderaManager(BaseApiResource):
     """
     return self._get("peers", ApiCmPeer, True, api_version=3)
 
-  def get_peer(self, name):
+  def get_peer(self, name, peer_type="REPLICATION"):
     """
     Retrieve a replication peer by name.
 
     @param name: The name of the peer.
+    @param peer_type: Added in v11. The type of the peer. Defaults to 'REPLICATION'.
     @return: The peer.
     @since: API v3
     """
-    return self._get("peers/" + name, ApiCmPeer, api_version=3)
+    params = self._get_peer_type_param(peer_type)
+    return self._get("peers/" + name, ApiCmPeer, params=params, api_version=3)
 
-  def test_peer_connectivity(self, name):
+  def test_peer_connectivity(self, name, peer_type="REPLICATION"):
     """
     Test connectivity for a replication peer.
 
     @param name: The name of the peer to test.
+    @param peer_type: Added in v11. The type of the peer to test. Defaults to 'REPLICATION'.
     @return: The command representing the test.
     @since: API v3
     """
-    return self._post("peers/%s/commands/test" % (name, ), ApiCommand,
+    params = self._get_peer_type_param(peer_type)
+    return self._post("peers/%s/commands/test" % name, ApiCommand, params=params,
         api_version=3)
 
   def get_all_hosts_config(self, view=None):
@@ -347,7 +441,8 @@ class ClouderaManager(BaseApiResource):
 
   def host_install(self, user_name, host_names, ssh_port=None, password=None,
           private_key=None, passphrase=None, parallel_install_count=None,
-          cm_repo_url=None, gpg_key_custom_url=None):
+          cm_repo_url=None, gpg_key_custom_url=None,
+          java_install_strategy=None, unlimited_jce=None):
     """
     Install Cloudera Manager Agent on a set of hosts.
 
@@ -381,6 +476,14 @@ class ClouderaManager(BaseApiResource):
                                http://archive-primary.cloudera.com/cm5/redhat/6/x86_64/cm/RPM-GPG-KEY-cloudera
                                Example for Ubuntu or other Debian based distributions:
                                http://archive.cloudera.com/debian/archive.key
+    @param java_install_strategy: Added in v8: Strategy to use for JDK installation. Valid values are 1.
+                                  AUTO (default): Cloudera Manager will install the JDK versions that are
+                                  required when the "AUTO" option is selected. Cloudera Manager may
+                                  overwrite any of the existing JDK installations. 2. NONE: Cloudera
+                                  Manager will not install any JDK when "NONE" option is selected. It
+                                  should be used if an existing JDK installation has to be used.
+    @param unlimited_jce: Added in v8: Flag for unlimited strength JCE policy files installation If
+                          unset, defaults to false
     @return: Information about the submitted command.
     @since: API v6
     """
@@ -403,6 +506,10 @@ class ClouderaManager(BaseApiResource):
      host_install_args['cmRepoUrl'] = cm_repo_url
     if gpg_key_custom_url:
      host_install_args['gpgKeyCustomUrl'] = gpg_key_custom_url
+    if java_install_strategy is not None:
+     host_install_args['javaInstallStrategy'] = java_install_strategy
+    if unlimited_jce:
+     host_install_args['unlimitedJCE'] = unlimited_jce
     return self._cmd('hostInstall', data=host_install_args)
 
   def begin_trial(self):
@@ -423,3 +530,33 @@ class ClouderaManager(BaseApiResource):
     @since: API v6
     """
     self._post("trial/end", None, api_version=6)
+
+  def import_cluster_template(self, api_cluster_template, add_repositories=False):
+    """
+    Create a cluster according to the provided template
+
+    @param api_cluster_template: cluster template to import
+    @param add_repositories: if true the parcels repositories in the cluster template will be added.
+    @return: Command handing cluster import
+    @since: API v12
+    """
+    return self._post("importClusterTemplate", ApiCommand, False, api_cluster_template, params=dict(addRepositories=add_repositories), api_version=12)
+
+  def delete_credentials(self):
+    """
+    Delete credentials for services configured with Kerberos.
+
+    @return: Information about the submitted command.
+    @since: API v17
+    """
+    return self._cmd('deleteCredentials', api_version=17)
+
+  def delete_credentials(self, mode):
+    """
+    Delete credentials for services configured with Kerberos.
+
+    @return: Information about the submitted command.
+    @param mode: "all" if all credentials need to be deleted, or "unused" if unused credentials are to be deleted.
+    @since: API v18
+    """
+    return self._cmd('deleteCredentials', mode, api_version=18)
