@@ -93,8 +93,10 @@ class ApiService(BaseApiResource):
     'configStalenessStatus'       : ROAttr(),
     'clientConfigStalenessStatus' : ROAttr(),
     'serviceUrl'                  : ROAttr(),
+    'roleInstancesUrl'            : ROAttr(),
     'maintenanceMode'             : ROAttr(),
     'maintenanceOwners'           : ROAttr(),
+    'entityStatus'                : ROAttr(),
   }
 
   def __init__(self, resource_root, name=None, type=None):
@@ -162,6 +164,36 @@ class ApiService(BaseApiResource):
   def get_activity(self, job_id):
     return self._get("activities/" + job_id, ApiActivity)
 
+  def list_watched_directories(self):
+    """
+    Returns a list of directories being watched by the Reports Manager.
+
+    @return: A list of directories being watched
+    @since: API v14
+    """
+    return self._get("watcheddir", ApiWatchedDir, ret_is_list=True, api_version=14)
+
+  def add_watched_directory(self, dir_path):
+    """
+    Adds a directory to the watching list.
+
+    @param dir_path: The path of the directory to be added to the watching list
+    @return: The added directory, or null if failed
+    @since: API v14
+    """
+    req = ApiWatchedDir(self._get_resource_root(), path=dir_path)
+    return self._post("watcheddir", ApiWatchedDir, data=req, api_version=14)
+
+  def remove_watched_directory(self, dir_path):
+    """
+    Removes a directory from the watching list.
+
+    @param dir_path: The path of the directory to be removed from the watching list
+    @return: The removed directory, or null if failed
+    @since: API v14
+    """
+    return self._delete("watcheddir/%s" % dir_path, ApiWatchedDir, api_version=14)
+
   def get_impala_queries(self, start_time, end_time, filter_str="", limit=100,
      offset=0):
     """
@@ -235,7 +267,7 @@ class ApiService(BaseApiResource):
     @since API v6
     """
     return self._get("impalaQueries/attributes", ApiImpalaQueryAttribute,
-        api_version=6)
+        ret_is_list=True, api_version=6)
 
   def create_impala_catalog_database(self):
     """
@@ -446,7 +478,7 @@ class ApiService(BaseApiResource):
     @since API v6
     """
     return self._get("yarnApplications/attributes", ApiYarnApplicationAttribute,
-        api_version=6)
+        ret_is_list=True, api_version=6)
 
   def create_yarn_job_history_dir(self):
     """
@@ -468,9 +500,11 @@ class ApiService(BaseApiResource):
 
   def collect_yarn_application_diagnostics(self, *application_ids):
     """
+    DEPRECATED: use create_yarn_application_diagnostics_bundle on the Yarn service. Deprecated since v10.
+
     Collects the Diagnostics data for Yarn applications.
 
-    @param application_ids: An array of strings containing the id's of the
+    @param application_ids: An array of strings containing the ids of the
                             yarn applications.
     @return: Reference to the submitted command.
     @since: API v8
@@ -478,6 +512,24 @@ class ApiService(BaseApiResource):
     args = dict(applicationIds = application_ids)
     return self._cmd('yarnApplicationDiagnosticsCollection', api_version=8, data=args)
 
+  def create_yarn_application_diagnostics_bundle(self, application_ids, ticket_number=None, comments=None):
+    """
+    Collects the Diagnostics data for Yarn applications.
+
+    @param application_ids: An array of strings containing the ids of the
+                            yarn applications.
+    @param ticket_number: If applicable, the support ticket number of the issue
+                          being experienced on the cluster.
+    @param comments: Additional comments
+    @return: Reference to the submitted command.
+    @since: API v10
+    """
+    args = dict(applicationIds = application_ids,
+                ticketNumber = ticket_number,
+                comments = comments)
+
+    return self._cmd('yarnApplicationDiagnosticsCollection', api_version=10, data=args)
+
   def get_config(self, view = None):
     """
     Retrieve the service's configuration.
@@ -731,6 +783,17 @@ class ApiService(BaseApiResource):
     """
     return self._role_cmd('hdfsBootstrapStandBy', role_names)
 
+  def finalize_metadata_upgrade(self, *role_names):
+    """
+    Finalize HDFS NameNode metadata upgrade. Should be done after doing
+    HDFS upgrade with full downtime (and not with rolling upgrade).
+
+    @param role_names: NameNodes for which to finalize the upgrade.
+    @return: List of submitted commands.
+    @since: API v3
+    """
+    return self._role_cmd('hdfsFinalizeMetadataUpgrade', role_names, api_version=3)
+
   def create_beeswax_warehouse(self):
     """
     DEPRECATED: use create_hive_warehouse on the Hive service. Deprecated since v3.
@@ -776,6 +839,22 @@ class ApiService(BaseApiResource):
     """
     return self._cmd('decommission', data=role_names)
 
+  def offline(self, *role_names, **options):
+    """
+    Offline roles in a service.
+
+    @param role_names: List of names of the roles to offline.
+    @param options: Dictionary of options. The following are recognized:
+        timeout - Offline timeout in seconds.
+                  Specify to override the default offline timeout of 4 hours.
+    @return: Reference to the submitted command.
+    @since: API v17
+    """
+    timeout = options.pop("timeout", None)
+    return self._cmd('offline', data=role_names,
+        params=timeout and dict(timeout=timeout) or dict(),
+        api_version=17)
+
   def recommission(self, *role_names):
     """
     Recommission roles in a service.
@@ -786,6 +865,21 @@ class ApiService(BaseApiResource):
     """
     return self._cmd('recommission', data=role_names)
 
+  def recommission_with_start(self, *role_names):
+    """
+    Recommission roles in a service. Roles are started before recommissioning.
+
+    Warning: Evolving. This method may change in the future and does not offer
+    standard compatibility guarantees.
+    Only support by HDFS.
+    Do not use without guidance from Cloudera.
+
+    @param role_names: Names of the roles to recommission.
+    @return: Reference to the submitted command.
+    @since: API v15
+    """
+    return self._cmd('recommissionWithStart', data=role_names, api_version=15)
+
   def deploy_client_config(self, *role_names):
     """
     Deploys client configuration to the hosts where roles are running.
@@ -1109,6 +1203,41 @@ class ApiService(BaseApiResource):
     )
     return self._cmd('oozieDisableHa', data=args, api_version=6)
 
+  def enable_sentry_ha(self, new_sentry_host_id, new_sentry_role_name=None,
+                       zk_service_name=None):
+    """
+    Enable high availability for Sentry.
+
+    @param new_sentry_server_host_id: ID of the host on which new Sentry Server will be added
+    @param new_sentry_server_rolename: Name of the new Sentry Server. This is an optional argument.
+                                       A default name will be generated if not specified.
+    @param zk_service_name: Name of the ZooKeeper service that will be used for Oozie HA.
+                            This is an optional parameter if the Oozie to ZooKeeper
+                            dependency is already set.
+    @return: Reference to the submitted command.
+    @since: API v18
+    """
+    args = dict(
+      newSentryHostId = new_sentry_host_id,
+      newSentryRoleName = new_sentry_role_name,
+      zkServiceName = zk_service_name
+    )
+    return self._cmd('enableSentryHa', data=args, api_version=18)
+
+  def disable_sentry_ha(self, active_name):
+    """
+    Disable high availability for Sentry
+
+    @param active_name: Name of the Sentry Server that will be active after
+                        high availability is disabled.
+    @return: Reference to the submitted command.
+    @since: API v18
+    """
+    args = dict(
+      activeName = active_name
+    )
+    return self._cmd('disableSentryHa', data=args, api_version=18)
+
   def failover_hdfs(self, active_name, standby_name, force=False):
     """
     Initiate a failover of an HDFS NameNode HA pair.
@@ -1196,6 +1325,16 @@ class ApiService(BaseApiResource):
     """
     return self._cmd('createSqoopUserDir', api_version=4)
 
+  def create_sqoop_database_tables(self):
+    """
+    Creates the Sqoop2 Server database tables in the configured database.
+    Will do nothing if tables already exist. Will not perform an upgrade.
+
+    @return: Reference to the submitted command.
+    @since: API v10
+    """
+    return self._cmd('sqoopCreateDatabaseTables', api_version=10)
+
   def upgrade_sqoop_db(self):
     """
     Upgrade Sqoop Database schema as part of a major version upgrade.
@@ -1250,10 +1389,32 @@ class ApiService(BaseApiResource):
     """
     Synchronize the Hue server's database.
 
-    @param servers: Name of Hue Server roles to synchronize.
+    @param servers: Name of Hue Server roles to synchronize. Not required starting with API v10.
     @return: List of submitted commands.
     """
-    return self._role_cmd('hueSyncDb', servers)
+
+    actual_version = self._get_resource_root().version
+    if actual_version < 10:
+      return self._role_cmd('hueSyncDb', servers)
+
+    return self._cmd('hueSyncDb', api_version=10)
+
+
+  def dump_hue_db(self):
+    """
+    Dump the Hue server's database; it can be loaded later.
+
+    @return: List of submitted commands.
+    """
+    return self._cmd('hueDumpDb', api_version=10)
+
+  def load_hue_db(self):
+    """
+    Load data into Hue server's database from a previous data dump.
+
+    @return: List of submitted commands.
+    """
+    return self._cmd('hueLoadDb', api_version=10)
 
   def lsof(self, *rolenames):
     """
@@ -1408,9 +1569,12 @@ class ApiService(BaseApiResource):
       alertOnFail=alert_on_fail, alertOnAbort=alert_on_abort)
 
     if self.type == 'HDFS':
-      if not isinstance(arguments, ApiHdfsReplicationArguments):
+      if isinstance(arguments, ApiHdfsCloudReplicationArguments):
+        schedule.hdfsCloudArguments = arguments
+      elif isinstance(arguments, ApiHdfsReplicationArguments):
+        schedule.hdfsArguments = arguments
+      else:
         raise TypeError, 'Unexpected type for HDFS replication argument.'
-      schedule.hdfsArguments = arguments
     elif self.type == 'HIVE':
       if not isinstance(arguments, ApiHiveReplicationArguments):
         raise TypeError, 'Unexpected type for Hive replication argument.'
@@ -1501,6 +1665,19 @@ class ApiService(BaseApiResource):
         params=dict(dryRun=dry_run),
         api_version=3)
 
+  def collect_replication_diagnostic_data(self, schedule_id, diagnostic_args, view=None):
+    """
+    Collect replication diagnostic data for the specified schedule id
+
+    @param schedule_id: The id of the schedule to collect diagnostic data.
+    @param diagnostic_args: Arguments for collecting diagnostic data.
+    @return: The command corresponding to the collect diagnostic data.
+    @since: API v11
+    """
+    return self._post("replications/%s/collectDiagnosticData" % schedule_id, ApiCommand,
+        data=diagnostic_args, params=view and dict(view=view) or None,
+        api_version=11)
+
   def create_snapshot_policy(self, policy):
     """
     Create a new snapshot policy for this service.
@@ -1588,9 +1765,22 @@ class ApiService(BaseApiResource):
     """
     return self._cmd('installOozieShareLib', api_version=3)
 
+  def create_oozie_embedded_database(self):
+    """
+    Create the Oozie Server Database. Only works with embedded postgresql
+    database. This command should usually be followed by a call to
+    create_oozie_db.
+
+    @return: Reference to the submitted command.
+    @since: API v10
+    """
+    return self._cmd('oozieCreateEmbeddedDatabase', api_version=10)
+
   def create_oozie_db(self):
     """
     Creates the Oozie Database Schema in the configured database.
+    This command does not create database. This command creates only tables
+    required by Oozie. To create database, please refer to create_oozie_embedded_database.
 
     @return: Reference to the submitted command.
     @since: API v2
@@ -1667,7 +1857,7 @@ class ApiService(BaseApiResource):
     """
     Create the Sentry Server Database. Only works with embedded postgresql
     database. This command should usually be followed by a call to
-    create_hive_metastore_tables.
+    create_sentry_database_tables.
 
     @return: Reference to the submitted command.
     @since: API v7
@@ -1740,6 +1930,17 @@ class ApiService(BaseApiResource):
     """
     return self._cmd('switchToMr2', api_version=6)
 
+  def finalize_rolling_upgrade(self):
+    """
+    Finalizes the rolling upgrade for HDFS by updating the NameNode
+    metadata permanently to the next version. Should be done after
+    doing a rolling upgrade to a CDH version >= 5.2.0.
+
+    @return: Reference to the submitted command.
+    @since: API v8
+    """
+    return self._cmd('hdfsFinalizeRollingUpgrade', api_version=8)
+
   def role_command_by_name(self, command_name, *role_names):
     """
     Executes a role command by name on the specified
@@ -1774,6 +1975,25 @@ class ApiService(BaseApiResource):
     return self._get("commandsByName", ApiCommandMetadata, True,
         api_version=6)
 
+  def create_yarn_cm_container_usage_input_dir(self):
+    """
+    Creates the HDFS directory where YARN container usage metrics are
+    stored by NodeManagers for CM to read and aggregate into app usage metrics.
+
+    @return: Reference to submitted command.
+    @since: API v13
+    """
+    return self._cmd('yarnCreateCmContainerUsageInputDirCommand', api_version=13)
+
+  def validate_hive_metastore_schema(self):
+    """
+    Validates the Hive metastore database schema.
+
+    @return: Reference to the submitted command.
+    @since: API v7
+    """
+    return self._cmd('hiveValidateMetastoreSchema', api_version=17)
+
 class ApiServiceSetupInfo(ApiService):
   _ATTRIBUTES = {
     'name'    : None,
@@ -1843,4 +2063,3 @@ class ApiServiceSetupInfo(ApiService):
     @since: API v7
     """
     return self._cmd('firstRun', None, api_version=7)
-
