@@ -501,6 +501,7 @@ class ApiCommand(BaseApiObject):
         'children'      : ROAttr(ApiCommand, is_api_list=True),
         'parent'        : ROAttr(ApiCommand),
         'resultDataUrl' : ROAttr(),
+        'canRetry'      : ROAttr(),
       }
     return cls._ATTRIBUTES
 
@@ -570,6 +571,16 @@ class ApiCommand(BaseApiObject):
     resp = self._get_resource_root().post(path)
     return ApiCommand.from_json_dict(resp, self._get_resource_root())
 
+  def retry(self):
+    """
+    Retry a failed or aborted command.
+
+    @return: A new ApiCommand object with the updated information.
+    """
+    path = self._path() + '/retry'
+    resp = self._get_resource_root().post(path)
+    return ApiCommand.from_json_dict(resp, self._get_resource_root())
+
 class ApiBulkCommandList(ApiList):
   _ATTRIBUTES = {
     'errors' : ROAttr(),
@@ -659,11 +670,27 @@ class ApiCmPeer(BaseApiObject):
       'url'       : None,
       'username'  : None,
       'password'  : None,
+      'type'      : None,
+      'clouderaManagerCreatedUser' : None,
     }
 
   def __str__(self):
     return "<ApiPeer>: %s (%s)" % (self.name, self.url)
 
+class ApiLicensedFeatureUsage(BaseApiObject):
+  _ATTRIBUTES = {
+    'totals'     : ROAttr(),
+    'clusters'  : ROAttr(),
+  }
+
+class ApiReplicationDiagnosticsCollectionArgs(BaseApiObject):
+  _ATTRIBUTES = {
+    'commands'        : ROAttr(),
+    'ticketNumber'    : ROAttr(),
+    'comments'        : ROAttr(),
+    'phoneHome'       : ROAttr(),
+  }
+
 class ApiHdfsReplicationArguments(BaseApiObject):
   _ATTRIBUTES = {
     'sourceService'             : Attr(ApiServiceRef),
@@ -671,8 +698,11 @@ class ApiHdfsReplicationArguments(BaseApiObject):
     'destinationPath'           : None,
     'mapreduceServiceName'      : None,
     'userName'                  : None,
+    'sourceUser'                : None,
     'numMaps'                   : None,
     'dryRun'                    : None,
+    'bandwidthPerMap'           : None,
+    'logPath'                   : None,
     'schedulerPoolName'         : None,
     'abortOnError'              : None,
     'preservePermissions'       : None,
@@ -680,8 +710,25 @@ class ApiHdfsReplicationArguments(BaseApiObject):
     'preserveReplicationCount'  : None,
     'removeMissingFiles'        : None,
     'skipChecksumChecks'        : None,
+    'skipListingChecksumChecks' : None,
+    'skipTrash'                 : None,
+    'replicationStrategy'       : None,
+    'preserveXAttrs'            : None,
+    'exclusionFilters'          : None,
   }
 
+class ApiHdfsCloudReplicationArguments(ApiHdfsReplicationArguments):
+  @classmethod
+  def _get_attributes(cls):
+    if not cls.__dict__.has_key('_ATTRIBUTES'):
+      attrs = {
+        'sourceAccount' : None,
+        'destinationAccount' : None,
+      }
+      attrs.update(ApiHdfsReplicationArguments._get_attributes())
+      cls._ATTRIBUTES = attrs
+    return cls._ATTRIBUTES
+
 class ApiHdfsReplicationResult(BaseApiObject):
   _ATTRIBUTES = {
     'progress'            : ROAttr(),
@@ -702,6 +749,9 @@ class ApiHdfsReplicationResult(BaseApiObject):
     'jobDetailsUri'       : ROAttr(),
     'dryRun'              : ROAttr(),
     'snapshottedDirs'     : ROAttr(),
+    'failedFiles'         : ROAttr(),
+    'runAsUser'           : ROAttr(),
+    'runOnSourceAsUser'   : ROAttr(),
   }
 
 class ApiHiveTable(BaseApiObject):
@@ -722,6 +772,15 @@ class ApiImpalaUDF(BaseApiObject):
   def __str__(self):
     return "<ApiImpalaUDF>: %s, %s" % (self.database, self.signature)
 
+class ApiHiveUDF(BaseApiObject):
+  _ATTRIBUTES = {
+    'database'  : ROAttr(),
+    'signature' : ROAttr(),
+  }
+
+  def __str__(self):
+    return "<ApiHiveUDF>: %s, %s" % (self.database, self.signature)
+
 class ApiHiveReplicationArguments(BaseApiObject):
   _ATTRIBUTES = {
     'sourceService' : Attr(ApiServiceRef),
@@ -731,6 +790,7 @@ class ApiHiveReplicationArguments(BaseApiObject):
     'replicateData' : None,
     'hdfsArguments' : Attr(ApiHdfsReplicationArguments),
     'dryRun'        : None,
+    'replicateImpalaMetadata' : None,
   }
 
 class ApiHiveReplicationResult(BaseApiObject):
@@ -739,10 +799,14 @@ class ApiHiveReplicationResult(BaseApiObject):
     'tables'                : ROAttr(ApiHiveTable),
     'impalaUDFCount'        : ROAttr(),
     'impalaUDFs'            : ROAttr(ApiImpalaUDF),
+    'hiveUDFCount'          : ROAttr(),
+    'hiveUDFs'              : ROAttr(ApiHiveUDF),
     'errorCount'            : ROAttr(),
     'errors'                : ROAttr(),
     'dataReplicationResult' : ROAttr(ApiHdfsReplicationResult),
     'dryRun'                : ROAttr(),
+    'runAsUser'             : ROAttr(),
+    'runOnSourceAsUser'     : ROAttr(),
     'phase'                 : ROAttr(),
   }
 
@@ -767,6 +831,7 @@ class ApiReplicationSchedule(BaseApiObject):
     'paused'          : None,
     'hdfsArguments'   : Attr(ApiHdfsReplicationArguments),
     'hiveArguments'   : Attr(ApiHiveReplicationArguments),
+    'hdfsCloudArguments'   : Attr(ApiHdfsCloudReplicationArguments),
     'alertOnStart'    : None,
     'alertOnSuccess'  : None,
     'alertOnFail'     : None,
@@ -774,6 +839,7 @@ class ApiReplicationSchedule(BaseApiObject):
     'id'              : ROAttr(),
     'nextRun'         : ROAttr(datetime.datetime),
     'history'         : ROAttr(ApiReplicationCommand),
+    'active'          : None
   }
 
 class ApiHBaseSnapshotPolicyArguments(BaseApiObject):
@@ -903,6 +969,7 @@ class ApiSnapshotPolicy(BaseApiObject):
   @ivar alert_on_success: whether to generate alerts on successful completion of snapshot creation/deletion activity.
   @ivar alert_on_fail: whether to generate alerts on failure of snapshot creation/deletion activity.
   @ivar alert_on_abort: whether to generate alerts on abort of snapshot creation/deletion activity.
+  @ivar paused: whether to run the policy on schedule
   @type hbaseArguments: ApiHBaseSnapshotPolicyArguments
   @ivar hbaseArguments: HBase specific arguments for the replication job.
   @type hdfsArguments: ApiHdfsSnapshotPolicyArguments
@@ -926,6 +993,7 @@ class ApiSnapshotPolicy(BaseApiObject):
     'alertOnSuccess'          : None,
     'alertOnFail'             : None,
     'alertOnAbort'            : None,
+    'paused'                  : None,
     'hbaseArguments'          : Attr(ApiHBaseSnapshotPolicyArguments),
     'hdfsArguments'           : Attr(ApiHdfsSnapshotPolicyArguments),
     'lastCommand'             : ROAttr(ApiSnapshotCommand),
@@ -966,15 +1034,17 @@ class ApiBatchResponseList(ApiList):
 
 class ApiConfig(BaseApiObject):
   _ATTRIBUTES = {
-    'name'              : None,
-    'value'             : None,
-    'required'          : ROAttr(),
-    'default'           : ROAttr(),
-    'displayName'       : ROAttr(),
-    'description'       : ROAttr(),
-    'relatedName'       : ROAttr(),
-    'validationState'   : ROAttr(),
-    'validationMessage' : ROAttr(),
+    'name'                         : None,
+    'value'                        : None,
+    'required'                     : ROAttr(),
+    'default'                      : ROAttr(),
+    'displayName'                  : ROAttr(),
+    'description'                  : ROAttr(),
+    'relatedName'                  : ROAttr(),
+    'sensitive'                    : ROAttr(),
+    'validationState'              : ROAttr(),
+    'validationMessage'            : ROAttr(),
+    'validationWarningsSuppressed' : ROAttr()
   }
 
   def __init__(self, resource_root, name=None, value=None):
@@ -1003,6 +1073,25 @@ class ApiImpalaQuery(BaseApiObject):
   def __str__(self):
     return "<ApiImpalaQuery>: %s" % (self.queryId)
 
+#
+# WatchedDirectories data types
+#
+
+class ApiWatchedDir(BaseApiObject):
+
+  _ATTRIBUTES = {
+    'path'  : None
+  }
+
+  def __str__(self):
+    return "<ApiWatchedDir>: %s" % (self.path)
+
+class ApiWatchedDirList(ApiList):
+
+  _ATTRIBUTES = {
+    'watchedDirs'  : ROAttr(ApiWatchedDir)
+  }
+  _MEMBER_CLASS = ApiWatchedDir
 
 class ApiImpalaQueryResponse(BaseApiObject):
 
@@ -1050,16 +1139,28 @@ class ApiMr2AppInformation(BaseApiObject):
 
 class ApiYarnApplication(BaseApiObject):
   _ATTRIBUTES = {
-    'applicationId'          : ROAttr(),
-    'name'                   : ROAttr(),
-    'user'                   : ROAttr(),
-    'startTime'              : ROAttr(datetime.datetime),
-    'endTime'                : ROAttr(datetime.datetime),
-    'pool'                   : ROAttr(),
-    'state'                  : ROAttr(),
-    'progress'               : ROAttr(),
-    'mr2AppInformation'      : ROAttr(ApiMr2AppInformation),
-    'attributes'             : ROAttr(),
+    'applicationId'                    : ROAttr(),
+    'name'                             : ROAttr(),
+    'user'                             : ROAttr(),
+    'startTime'                        : ROAttr(datetime.datetime),
+    'endTime'                          : ROAttr(datetime.datetime),
+    'pool'                             : ROAttr(),
+    'state'                            : ROAttr(),
+    'progress'                         : ROAttr(),
+    'mr2AppInformation'                : ROAttr(ApiMr2AppInformation),
+    'attributes'                       : ROAttr(),
+    'allocatedMB'                      : ROAttr(),
+    'allocatedVCores'                  : ROAttr(),
+    'runningContainers'                : ROAttr(),
+    'applicationTags'                  : ROAttr(),
+    'allocatedMemorySeconds'           : ROAttr(),
+    'allocatedVcoreSeconds'            : ROAttr(),
+    'containerUsedMemorySeconds'       : ROAttr(),
+    'containerUsedCpuSeconds'          : ROAttr(),
+    'containerUsedVcoreSeconds'        : ROAttr(),
+    'containerAllocatedMemorySeconds'  : ROAttr(),
+    'containerAllocatedVcoreSeconds'   : ROAttr(),
+    'containerUsedMemoryMax'           : ROAttr(),
   }
 
   def __str__(self):
@@ -1093,6 +1194,107 @@ class ApiYarnApplicationAttribute(BaseApiObject):
   def __str__(self):
     return "<ApiYarnApplicationAttribute> %s" % name
 
+class ApiTimeSeriesRequest(BaseApiObject):
+  _ATTRIBUTES = {
+      'query'                : None,
+      'from'                 : None,
+      'to'                   : None,
+      'contentType'          : None,
+      'desiredRollup'        : None,
+      'mustUseDesiredRollup' : None
+    }
+
+  def __str__(self):
+    return "<ApiTimeSeriesRequest>: %s" % (self.query)
+
+class ApiProductVersion(BaseApiObject):
+  _ATTRIBUTES = {
+    'version'       : None,
+    'product'       : None,
+  }
+
+class ApiClusterTemplateConfig(BaseApiObject):
+  _ATTRIBUTES = {
+    'name'        : None,
+    'value'       : None,
+    'ref'         : None,
+    'variable'    : None,
+    'autoConfig'  : None,
+  }
+
+class ApiClusterTemplateRoleConfigGroup(BaseApiObject):
+  _ATTRIBUTES = {
+    'refName'        : None,
+    'roleType'       : None,
+    'base'           : None,
+    'displayName'    : None,
+    'configs'        : Attr(ApiClusterTemplateConfig),
+  }
+
+class ApiClusterTemplateRole(BaseApiObject):
+  _ATTRIBUTES = {
+    'refName'           : None,
+    'roleType'          : None,
+  }
+
+class ApiClusterTemplateHostTemplate(BaseApiObject):
+  _ATTRIBUTES = {
+    'refName'                   : None,
+    'cardinality'               : None,
+    'roleConfigGroupsRefNames'  : None,
+  }
+
+class ApiClusterTemplateHostInfo(BaseApiObject):
+  _ATTRIBUTES = {
+    'hostName'                  : None,
+    'hostNameRange'             : None,
+    'rackId'                    : None,
+    'hostTemplateRefName'       : None,
+    'roleRefNames'              : None,
+  }
+
+class ApiClusterTemplateVariable(BaseApiObject):
+  _ATTRIBUTES = {
+    'name'         : None,
+    'value'        : None,
+  }
+
+class ApiClusterTemplateRoleConfigGroupInfo(BaseApiObject):
+  _ATTRIBUTES = {
+    'rcgRefName'  : None,
+    'name'        : None,
+  }
+
+class ApiClusterTemplateInstantiator(BaseApiObject):
+  _ATTRIBUTES = {
+    'clusterName'      : None,
+    'hosts'            : Attr(ApiClusterTemplateHostInfo),
+    'variables'        : Attr(ApiClusterTemplateVariable),
+    'roleConfigGroups' : Attr(ApiClusterTemplateRoleConfigGroupInfo),
+  }
+
+class ApiClusterTemplateService(BaseApiObject):
+  _ATTRIBUTES = {
+    'refName'           : None,
+    'serviceType'       : None,
+    'serviceConfigs'    : Attr(ApiClusterTemplateConfig),
+    'roleConfigGroups'  : Attr(ApiClusterTemplateRoleConfigGroup),
+    'displayName'       : None,
+    'roles'             : Attr(ApiClusterTemplateRole),
+  }
+
+class ApiClusterTemplate(BaseApiObject):
+  _ATTRIBUTES = {
+    'cdhVersion'    : None,
+    'displayName'   : None,
+    'cmVersion'     : None,
+    "repositories"  : None,
+    'products'      : Attr(ApiProductVersion),
+    'services'      : Attr(ApiClusterTemplateService),
+    'hostTemplates' : Attr(ApiClusterTemplateHostTemplate),
+    'instantiator'  : Attr(ApiClusterTemplateInstantiator),
+  }
+
 def config_to_api_list(dic):
   """
   Converts a python dictionary into a list containing the proper
